#!/usr/bin/env python
"""
_GetOpenBlocks_

Oracle implementation of DBS3Buffer.GetOpenBlocks
"""

from WMComponent.DBS3Buffer.MySQL.GetOpenBlocks import GetOpenBlocks as MySQLGetOpenBlocks

class GetOpenBlocks(MySQLGetOpenBlocks):
    """
    Identical to MySQL version

    """
