#!/usr/bin/env python
"""
_AddBulkParentage_

Oracle implementation of Files.AddBulkParentage
"""




from WMCore.WMBS.MySQL.Files.AddBulkParentage import AddBulkParentage as ParentageMySQL

class AddBulkParentage(ParentageMySQL):
    pass
