#!/usr/bin/env python
"""
_GetBlock_

Oracle implementation of DBSBufferFiles.GetBlock
"""




from WMComponent.DBS3Buffer.MySQL.DBSBufferFiles.GetBlock import GetBlock as MySQLGetBlock

class GetBlock(MySQLGetBlock):
    """
    _GetBlock_

    Oracle implementation of DBSBufferFiles.GetBlock
    """
    pass
