#!/usr/bin/env python
"""
_LoadForTaskArchiver_

Oracle implementation of Jobs.LoadForTaskArchiver

Created on Sep 5, 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Jobs.LoadForTaskArchiver import LoadForTaskArchiver as MySQLLoadForTaskArchiver

class LoadForTaskArchiver(MySQLLoadForTaskArchiver):
    pass
