#!/usr/bin/env python
"""
_GetJobGroups_

Oracle implementation of Subscriptions.GetJobGroups
"""




from WMCore.WMBS.MySQL.Subscriptions.GetJobGroups import GetJobGroups as GetJobGroupsMySQL

class GetJobGroups(GetJobGroupsMySQL):
    pass
