# -*- coding: utf-8 -*-
"""
_SetRunningSlots_

Oracle implementation of Locations.SetRunningSlots
Created on Mon Jun 18 14:08:58 2012

@author: dballest
"""

from WMCore.WMBS.MySQL.Locations.SetRunningSlots import SetRunningSlots as MySQLSetRunningSlots

class SetRunningSlots(MySQLSetRunningSlots):
    """
    Same as MySQL version
    """
    pass
