#!/usr/bin/env python
"""
_AddWorkUnits_

Oracle implementation of Jobs.AddWorkUnits
"""
from __future__ import absolute_import, division, print_function

from WMCore.WMBS.MySQL.Jobs.AddWorkUnits import AddWorkUnits as AddWorkUnitsMySQL


class AddWorkUnits(AddWorkUnitsMySQL):
    """
    _AddWorkUnits_

    Oracle implementation of Jobs.AddWorkUnits
    """

    pass
